/***************************************************************************************************************:')

U3Config.c

Main functions for the Labjack U3 handling.

Emilie Jude
Nassima Bouchelit
Fabrice Le Bars

Modified by Arunas Mazeika and Stphane Bazeille
Some functions and parts of functions are from the file ue9.c and the examples provided by www.labjack.com

2006-2007

Version 1.0.0

***************************************************************************************************************:)*/

#include "U3Config.h"

int ErrorHandler (LJ_ERROR lngErrorcode, long lngLineNumber, long lngIteration)	{

	char err[255];

	if (lngErrorcode != LJE_NOERROR)	{
		ErrorToString(lngErrorcode,err);
		fprintf(stdout, "Error number = %d\n", lngErrorcode);
		fprintf(stdout, "Error string = %s\n", err);
		fprintf(stdout, "Source line number = %d\n", lngLineNumber);
		fprintf(stdout, "Iteration = %d\n\n", lngIteration);
		if(lngErrorcode > LJE_MIN_GROUP_ERROR)	{
		}
		return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;
}

/*
Opens the first found LabJack U3.
LJ_HANDLE* pHandle : (OUT) pointer to an identifier representing the device
Returns : EXIT_SUCCESS or EXIT_FAILURE if there is an error
*/
int U3Open(LJ_HANDLE* pHandle)	{

	LJ_ERROR lngErrorcode; // Variable used to store the error code
	lngErrorcode = OpenLabJack(LJ_dtU3, LJ_ctUSB, "1", 1, pHandle);
	ErrorHandler(lngErrorcode, __LINE__, 0);
	return EXIT_SUCCESS;
}

/*  
Resets all U3 ports to their default configuration.  
This is recommended before using the U3 to ensure its ports are in the proper state. 
LJ_HANDLE pHandle : (IN) identifier representing the device
Returns : EXIT_SUCCESS or EXIT_FAILURE if there is an error
*/
int U3Reset(LJ_HANDLE lngHandle) {

	LJ_ERROR lngErrorcode; // Variable used to store the error code
	lngErrorcode = ePut(lngHandle, LJ_ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	ErrorHandler(lngErrorcode, __LINE__, 0);
	return EXIT_SUCCESS;
}

/*
Outputs a PWM (freq_syst = 24 MHz and freq_PWM = freq_syst/(timer_clock_divisor * 2^16)) on FIO0.
LJ_HANDLE pHandle : (IN) identifier representing the device
int timer_clock_divisor : (IN) divisor of the frequency
int timer_duty_cycle0 : (IN) value setting the duty cycle of Timer0 (between 0 and 65535)
Returns : EXIT_SUCCESS or EXIT_FAILURE if there is an error
*/
int U3ConfigTimers(LJ_HANDLE lngHandle, int timer_clock_divisor, int timer_duty_cycle_init)	{

	// Used to get the results of the different requests
	long lngGetNextIteration = 0, lngIOType = 0, lngChannel = 0; 
	double dblValue = 0;
	LJ_ERROR lngErrorcode; // Variable used to store the error code

	// First requests to configure the timer and counter. These will be
	// done with and add/go/get block.
	// Sets the timer/counter pin offset to 0, which will put the first
	// timer/counter on FIO0.
	lngErrorcode = AddRequest (lngHandle,  LJ_ioPUT_CONFIG, LJ_chTIMER_COUNTER_PIN_OFFSET, 0, 0, 0);
	ErrorHandler(lngErrorcode, __LINE__, 0);
	// Enables two timers. It will use FIO0 and FIO1
	AddRequest (lngHandle, LJ_ioPUT_CONFIG, LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
	// Makes sure Counter0 is disabled.
	AddRequest (lngHandle, LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
	// Makes sure Counter1 is disabled.
	AddRequest (lngHandle, LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);

	// All output timers use the same timer clock, configured here. The
	// base clock is set to 24MHZ_DIV, meaning that the clock divisor
	// is supported and Counter0 is not available.
	//AddRequest (lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc24MHZ_DIV, 0, 0);
	AddRequest (lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_BASE, LJ_tc4MHZ_DIV, 0, 0);
	// Sets the timer clock divisor
	AddRequest (lngHandle, LJ_ioPUT_CONFIG, LJ_chTIMER_CLOCK_DIVISOR, timer_clock_divisor, 0, 0);
	
	// Configures Timer0 as 16-bit PWM.
	AddRequest (lngHandle, LJ_ioPUT_TIMER_MODE, 0, LJ_tmPWM16, 0, 0);
	// Configures Timer1 as 16-bit PWM.
	AddRequest (lngHandle, LJ_ioPUT_TIMER_MODE, 1, LJ_tmPWM16, 0, 0);

	// Initializes the 16-bit PWM on Timer0 with a duty cycle .
	AddRequest (lngHandle, LJ_ioPUT_TIMER_VALUE, 0, timer_duty_cycle_init, 0, 0);
	// Initializes the 16-bit PWM on Timer1 with a duty cycle .
	AddRequest (lngHandle, LJ_ioPUT_TIMER_VALUE, 1, timer_duty_cycle_init, 0, 0);

	// Executes the requests.
	lngErrorcode = GoOne (lngHandle);
	ErrorHandler(lngErrorcode, __LINE__, 0);

	// Gets all the results just to check for errors.
	lngErrorcode = GetFirstResult(lngHandle, &lngIOType, &lngChannel, &dblValue, 0, 0);
	ErrorHandler(lngErrorcode, __LINE__, 0);
	lngGetNextIteration = 0;	// Used by the error handling function.
	while (lngErrorcode < LJE_MIN_GROUP_ERROR)	{

		lngErrorcode = GetNextResult(lngHandle, &lngIOType, &lngChannel, &dblValue, 0, 0);
		if (lngErrorcode != LJE_NO_MORE_DATA_AVAILABLE)	{
			ErrorHandler(lngErrorcode, __LINE__, lngGetNextIteration);
		}

		lngGetNextIteration++;
	}
	return EXIT_SUCCESS;
}

/*
Modify the timer_dutyc ycle .
LJ_HANDLE* pHandle : (IN,OUT) pointer to an identifier representing the device
Returns : EXIT_SUCCESS or EXIT_FAILURE if there is an error
*/
int U3ModifyDutyCycle(LJ_HANDLE lngHandle, int timer_duty_cycle, int num_timer){
	ePut(lngHandle, LJ_ioPUT_TIMER_VALUE, num_timer, timer_duty_cycle, 0);
	return EXIT_SUCCESS;
}

/*
Closes the device.
LJ_HANDLE* pHandle : (IN,OUT) pointer to an identifier representing the device
Returns : EXIT_SUCCESS or EXIT_FAILURE if there is an error
*/
int U3Close(LJ_HANDLE* pHandle)	{
	return EXIT_SUCCESS;
}


